#!/usr/bin/env bash

# Licensed to the LF AI & Data foundation under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

SOURCE="${BASH_SOURCE[0]}"
while [ -h "$SOURCE" ]; do # resolve $SOURCE until the file is no longer a symlink
  DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
  SOURCE="$(readlink "$SOURCE")"
  [[ $SOURCE != /* ]] && SOURCE="$DIR/$SOURCE" # if $SOURCE was a relative symlink, we need to resolve it relative to the path where the symlink file was located
done
ROOT_DIR="$( cd -P "$( dirname "$SOURCE" )/.." && pwd )"

CPP_SRC_DIR="${ROOT_DIR}/internal/core"

pushd $CPP_SRC_DIR
./run_clang_format.sh .
popd

if [[ $(uname -s) == "Darwin" ]]; then
  if ! brew --prefix --installed grep >/dev/null 2>&1; then
        brew install grep
  fi
  export PATH="/usr/local/opt/grep/libexec/gnubin:$PATH"
fi

check_result=$(git status | grep -E "*\.h|*\.hpp|*\.cc|*\.cpp")
echo "check_result: $check_result"
if test -z "$check_result"; then
  exit 0
else
  echo "The cpp files are not formatted, please use internal/core/run_clang_format.sh"
  exit 1
fi
